@@ -48,8 +48,7 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 /** Mode for the ArrayDirectory class. */
 enum class ArrayDirectoryMode {
@@ -456,13 +455,6 @@ class ArrayDirectory {
   /** Returns the URI for a vacuum file. */
   URI get_vacuum_uri(const URI& fragment_uri) const;
 
-  /**
-   * The new fragment name is computed
-   * as `__<first_URI_timestamp>_<last_URI_timestamp>_<uuid>`.
-   */
-  std::string compute_new_fragment_name(
-      const URI& first, const URI& last, format_version_t format_version) const;
-
   /** Returns `true` if `load` has been run. */
   bool loaded() const;
 
@@ -772,6 +764,13 @@ class ArrayDirectory {
   Status compute_array_schema_uris(
       const std::vector<URI>& array_schema_dir_uris);
 
+  /**
+   * Select the URI to use for the latest array schema.
+   *
+   * @return URI The latest array schema URI to use.
+   */
+  URI select_latest_array_schema_uri();
+
   /**
    * Checks if a fragment overlaps with the array directory timestamp
    * range. Overlap is partial or full depending on the consolidation
@@ -832,7 +831,6 @@ class ArrayDirectory {
       MemoryTracker& memory_tracker) const;
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_ARRAY_DIRECTORY_H
