@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2023 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -42,11 +42,13 @@
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/curl/curl_init.h"
 #include "tiledb/sm/filesystem/s3_thread_pool_executor.h"
+#include "tiledb/sm/filesystem/ssl_config.h"
 #include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/stats/global_stats.h"
 #include "tiledb/sm/stats/stats.h"
 #include "uri.h"
 
+#undef GetObject
 #include <aws/core/Aws.h>
 #include <aws/core/auth/AWSCredentialsProviderChain.h>
 #include <aws/core/client/ClientConfiguration.h>
@@ -80,14 +82,194 @@
 #include <vector>
 
 using namespace tiledb::common;
+using tiledb::common::filesystem::directory_entry;
 
-namespace tiledb {
+namespace tiledb::sm {
 
-namespace common::filesystem {
-class directory_entry;
-}
+class TileDBS3Client;
 
-namespace sm {
+/**
+ * The s3-specific configuration parameters.
+ *
+ * @note The member variables' default declarations have not yet been moved
+ * from the Config declaration into this struct.
+ * @note Not all vfs.s3 config parameters are present in this struct.
+ */
+struct S3Parameters {
+  /** Stores parsed custom headers from config. */
+  using Headers = std::unordered_map<std::string, std::string>;
+
+  S3Parameters() = delete;
+
+  S3Parameters(const Config& config)
+      : region_(config.get<std::string>("vfs.s3.region").value_or(""))
+      , aws_access_key_id_(config.get<std::string>(
+            "vfs.s3.aws_access_key_id", Config::must_find))
+      , aws_secret_access_key_(config.get<std::string>(
+            "vfs.s3.aws_secret_access_key", Config::must_find))
+      , aws_session_token_(config.get<std::string>(
+            "vfs.s3.aws_session_token", Config::must_find))
+      , aws_role_arn_(
+            config.get<std::string>("vfs.s3.aws_role_arn", Config::must_find))
+      , aws_external_id_(config.get<std::string>(
+            "vfs.s3.aws_external_id", Config::must_find))
+      , aws_load_frequency_(config.get<std::string>(
+            "vfs.s3.aws_load_frequency", Config::must_find))
+      , aws_session_name_(
+            config.get<std::string>("vfs.s3.aws_session_name").value())
+      , scheme_(config.get<std::string>("vfs.s3.scheme", Config::must_find))
+      , endpoint_override_(config.get<std::string>(
+            "vfs.s3.endpoint_override", Config::must_find))
+      , use_virtual_addressing_(config.get<bool>(
+            "vfs.s3.use_virtual_addressing", Config::must_find))
+      , skip_init_(config.get<bool>("vfs.s3.skip_init", Config::must_find))
+      , use_multipart_upload_(
+            config.get<bool>("vfs.s3.use_multipart_upload", Config::must_find))
+      , max_parallel_ops_(
+            config.get<uint64_t>("vfs.s3.max_parallel_ops", Config::must_find))
+      , multipart_part_size_(config.get<uint64_t>(
+            "vfs.s3.multipart_part_size", Config::must_find))
+      , connect_timeout_ms_(
+            config.get<int64_t>("vfs.s3.connect_timeout_ms", Config::must_find))
+      , connect_max_tries_(
+            config.get<int64_t>("vfs.s3.connect_max_tries", Config::must_find))
+      , connect_scale_factor_(config.get<int64_t>(
+            "vfs.s3.connect_scale_factor", Config::must_find))
+      , custom_headers_(load_headers(config))
+      , logging_level_(
+            config.get<std::string>("vfs.s3.logging_level", Config::must_find))
+      , request_timeout_ms_(
+            config.get<int64_t>("vfs.s3.request_timeout_ms", Config::must_find))
+      , requester_pays_(
+            config.get<bool>("vfs.s3.requester_pays", Config::must_find))
+      , proxy_host_(
+            config.get<std::string>("vfs.s3.proxy_host", Config::must_find))
+      , proxy_port_(
+            config.get<uint32_t>("vfs.s3.proxy_port", Config::must_find))
+      , proxy_scheme_(
+            config.get<std::string>("vfs.s3.proxy_scheme", Config::must_find))
+      , proxy_username_(
+            config.get<std::string>("vfs.s3.proxy_username", Config::must_find))
+      , proxy_password_(
+            config.get<std::string>("vfs.s3.proxy_password", Config::must_find))
+      , no_sign_request_(
+            config.get<bool>("vfs.s3.no_sign_request", Config::must_find))
+      , sse_algorithm_(config.get<std::string>("vfs.s3.sse", Config::must_find))
+      , sse_kms_key_id_(
+            sse_algorithm_ == "kms" ?
+                config.get<std::string>("vfs.s3.sse_kms_key_id").value() :
+                "")
+      , bucket_acl_str_(config.get<std::string>(
+            "vfs.s3.bucket_canned_acl", Config::must_find))
+      , object_acl_str_(config.get<std::string>(
+            "vfs.s3.object_canned_acl", Config::must_find))
+      , config_source_(config.get<std::string>(
+            "vfs.s3.config_source", Config::must_find)){};
+
+  ~S3Parameters() = default;
+
+  /** Load all custom headers from the given config.  */
+  static Headers load_headers(const Config& cfg);
+
+  /** The AWS region. */
+  std::string region_;
+
+  /** Set the AWS_ACCESS_KEY_ID. */
+  std::string aws_access_key_id_;
+
+  /** Set the AWS_SECRET_ACCESS_KEY. */
+  std::string aws_secret_access_key_;
+
+  /** Set the AWS_SESSION_TOKEN. */
+  std::string aws_session_token_;
+
+  /** Set the AWS_ROLE_ARN. Determines the role that we want to assume. */
+  std::string aws_role_arn_;
+
+  /** Set the AWS_EXTERNAL_ID. Third party access ID when assuming a role. */
+  std::string aws_external_id_;
+
+  /** Set the AWS_LOAD_FREQUENCY. Session time limit when assuming a role. */
+  std::string aws_load_frequency_;
+
+  /** Optional. Set the AWS_SESSION_NAME. Session name when assuming a role. */
+  std::string aws_session_name_;
+
+  /** The S3 scheme (`http` or `https`), if S3 is enabled. */
+  std::string scheme_;
+
+  /** The S3 endpoint, if S3 is enabled. */
+  std::string endpoint_override_;
+
+  /** Whether or not to use virtual addressing. */
+  bool use_virtual_addressing_;
+
+  /** Skip Aws::InitAPI for the S3 layer. */
+  bool skip_init_;
+
+  /** Whether or not to use multipart upload. */
+  bool use_multipart_upload_;
+
+  /** The maximum number of parallel operations issued. */
+  uint64_t max_parallel_ops_;
+
+  /** The part size (in bytes) used in S3 multipart writes. */
+  uint64_t multipart_part_size_;
+
+  /** The connection timeout in ms. Any `long` value is acceptable. */
+  int64_t connect_timeout_ms_;
+
+  /** The maximum tries for a connection. Any `long` value is acceptable. */
+  int64_t connect_max_tries_;
+
+  /** The scale factor for exponential backoff when connecting to S3. */
+  int64_t connect_scale_factor_;
+
+  /** Custom headers to add to all s3 requests. */
+  Headers custom_headers_;
+
+  /** Process-global AWS SDK logging level. */
+  std::string logging_level_;
+
+  /** The request timeout in ms. */
+  int64_t request_timeout_ms_;
+
+  /** If true, the requester pays for S3 access charges. */
+  bool requester_pays_;
+
+  /** The S3 proxy host. */
+  std::string proxy_host_;
+
+  /** The S3 proxy port. */
+  uint32_t proxy_port_;
+
+  /** The S3 proxy scheme. */
+  std::string proxy_scheme_;
+
+  /** The S3 proxy username. Not serialized by tiledb_config_save_to_file. */
+  std::string proxy_username_;
+
+  /** The S3 proxy password. */
+  std::string proxy_password_;
+
+  /** Make unauthenticated requests to s3. */
+  bool no_sign_request_;
+
+  /** The server-side encryption algorithm to use. "aes256" or "kms". */
+  std::string sse_algorithm_;
+
+  /** The server-side encryption key to use with the kms algorithm. */
+  std::string sse_kms_key_id_;
+
+  /** Names of values found in Aws::S3::Model::BucketCannedACL enumeration. */
+  std::string bucket_acl_str_;
+
+  /** Names of values found in Aws::S3::Model::ObjectCannedACL enumeration. */
+  std::string object_acl_str_;
+
+  /** Force S3 SDK to only load config options from a set source. */
+  std::string config_source_;
+};
 
 /**
  * This class implements the various S3 filesystem functions. It also
@@ -103,8 +285,14 @@ class S3 {
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
 
-  /** Constructor. */
-  S3();
+  /**
+   * Constructor.
+   *
+   * @param parent_stats The parent stats to inherit from.
+   * @param thread_pool The parent VFS thread pool.
+   * @param config Configuration parameters.
+   */
+  S3(stats::Stats* parent_stats, ThreadPool* thread_pool, const Config& config);
 
   /** Destructor. */
   ~S3();
@@ -116,19 +304,6 @@ class S3 {
   /*                 API               */
   /* ********************************* */
 
-  /**
-   * Initializes and connects an S3 client.
-   *
-   * @param parent_stats The parent stats.
-   * @param config Configuration parameters.
-   * @param thread_pool The parent VFS thread pool.
-   * @return Status
-   */
-  Status init(
-      stats::Stats* parent_stats,
-      const Config& config,
-      ThreadPool* thread_pool);
-
   /**
    * Creates a bucket.
    *
@@ -679,6 +854,9 @@ class S3 {
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /* The S3 configuration parameters. */
+  S3Parameters s3_params_;
+
   /**
    * A libcurl initializer instance. This should remain
    * the first member variable to ensure that libcurl is
@@ -696,7 +874,7 @@ class S3 {
    * The lazily-initialized S3 client. This is mutable so that nominally const
    * functions can call init_client().
    */
-  mutable shared_ptr<Aws::S3::S3Client> client_;
+  mutable shared_ptr<TileDBS3Client> client_;
 
   /** The AWS credetial provider. */
   mutable shared_ptr<Aws::Auth::AWSCredentialsProvider> credentials_provider_;
@@ -726,39 +904,18 @@ class S3 {
   /** Protects 'multipart_upload_states_'. */
   mutable RWLock multipart_upload_rwlock_;
 
-  /** The maximum number of parallel operations issued. */
-  uint64_t max_parallel_ops_;
-
-  /** The length of a non-terminal multipart part. */
-  uint64_t multipart_part_size_;
-
   /** File buffers used in the multi-part uploads. */
   std::unordered_map<std::string, Buffer*> file_buffers_;
 
-  /** The AWS region this instance was initialized with. */
-  std::string region_;
-
   /** Pointer to thread pool owned by parent VFS instance. */
   ThreadPool* vfs_thread_pool_;
 
-  /** Whether or not to use virtual addressing. */
-  bool use_virtual_addressing_;
-
-  /** Whether or not to use multipart upload. */
-  bool use_multipart_upload_;
-
-  /** Config stored from init for lazy client_init. */
-  Config config_;
-
   /** Set the request payer for a s3 request. */
   Aws::S3::Model::RequestPayer request_payer_;
 
   /** The server-side encryption algorithm. */
   Aws::S3::Model::ServerSideEncryption sse_;
 
-  /** The server-side encryption kms key. */
-  std::string sse_kms_key_id_;
-
   /** Protects file_buffers map */
   std::mutex file_buffers_mtx_;
 
@@ -768,6 +925,9 @@ class S3 {
   /** If !NOT_SET assign to bucket requests supporting SetACL() */
   Aws::S3::Model::BucketCannedACL bucket_canned_acl_;
 
+  /** Support the old s3 configuration values if configured by user. */
+  SSLConfig ssl_config_;
+
   friend class VFS;
 
   /* ********************************* */
@@ -778,6 +938,12 @@ class S3 {
    * Initializes the client, if it has not already been initialized.
    *
    * @return Status
+   *
+   * @note As currently implemented, this function is a direct obstacle of
+   * class C.41 compliance. This function is invoked by each function within
+   * the s3 class, giving the appearance that it could simply be moved into the
+   * constructor. This is NOT the case and would be breaking behavior until the
+   * class further matures to handle sessions.
    */
   Status init_client() const;
 
@@ -1026,8 +1192,7 @@ class S3 {
       const URI& attribute_uri, const std::string& chunk_name);
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // HAVE_S3
 #endif  // TILEDB_S3_H
