@@ -47,8 +47,7 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class GroupDetailsException : public StatusException {
  public:
@@ -96,10 +95,8 @@ class Group {
 
   /**
    * Clear a group
-   *
-   * @return
    */
-  Status clear();
+  void clear();
 
   /**
    * Deletes data from and closes a group opened in MODIFY_EXCLUSIVE mode.
@@ -301,23 +298,9 @@ class Group {
   /**
    * Function to generate a URL of a detail file
    *
-   * @return tuple of status and uri
+   * @return uri
    */
-  tuple<Status, optional<URI>> generate_detail_uri() const;
-
-  /**
-   * Have changes been applied to a group in write mode
-   * @return changes_applied_
-   */
-  bool changes_applied() const;
-
-  /**
-   * Set changes applied, only used in serialization
-   * @param changes_applied should changes be considered to be applied? If so
-   * then this will enable writes from a deserialized group
-   *
-   */
-  void set_changes_applied(bool changes_applied);
+  URI generate_detail_uri() const;
 
   /**
    * Get count of members
@@ -442,9 +425,6 @@ class Group {
   /** Mutex for thread safety. */
   mutable std::mutex mtx_;
 
-  /* Were changes applied and is a write is required */
-  bool changes_applied_;
-
   /* ********************************* */
   /*         PROTECTED METHODS         */
   /* ********************************* */
@@ -453,15 +433,7 @@ class Group {
    * Load group metadata, handles remote groups vs non-remote groups
    */
   void load_metadata();
-
-  /**
-   * Generate new name in the form of timestmap_timestamp_uuid
-   *
-   * @return tuple of status and optional string
-   */
-  tuple<Status, optional<std::string>> generate_name() const;
 };
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_GROUP_H
