@@ -64,6 +64,60 @@ class ReaderBase : public StrategyBase {
   /*          TYPE DEFINITIONS         */
   /* ********************************* */
 
+  class NameToLoad {
+   public:
+    /* ********************************* */
+    /*          STATIC FUNCTIONS         */
+    /* ********************************* */
+
+    /** Return a NameToLoad vector from a string vector. */
+    static std::vector<ReaderBase::NameToLoad> from_string_vec(
+        const std::vector<std::string>& names_to_load) {
+      std::vector<ReaderBase::NameToLoad> ret;
+      ret.reserve(names_to_load.size());
+      for (auto& name : names_to_load) {
+        ret.emplace_back(name);
+      }
+
+      return ret;
+    }
+
+    /* ********************************* */
+    /*     CONSTRUCTORS & DESTRUCTORS    */
+    /* ********************************* */
+
+    /** Constructor. */
+    NameToLoad(std::string name, bool validity_only = false)
+        : name_(name)
+        , validity_only_(validity_only) {
+    }
+
+    /* ********************************* */
+    /*          PUBLIC METHODS           */
+    /* ********************************* */
+
+    /** @returns the name to load. */
+    const std::string& name() const {
+      return name_;
+    }
+
+    /** @returns if the field needs to be loaded for validity only. */
+    bool validity_only() const {
+      return validity_only_;
+    }
+
+   private:
+    /* ********************************* */
+    /*        PRIVATE ATTRIBUTES         */
+    /* ********************************* */
+
+    /** Field name to load. */
+    const std::string name_;
+
+    /** Load validity only for the field. */
+    const bool validity_only_;
+  };
+
   /** The state for a read query. */
   struct ReadState {
     /**
@@ -291,6 +345,35 @@ class ReaderBase : public StrategyBase {
   /*         PROTECTED METHODS         */
   /* ********************************* */
 
+  /** Returns wether the field is for aggregation only or not. */
+  bool aggregate_only(const std::string& name) const {
+    if (qc_loaded_attr_names_set_.count(name) != 0) {
+      return false;
+    }
+
+    if (buffers_.count(name) != 0) {
+      return false;
+    }
+
+    return true;
+  }
+
+  /** Returns wether the field is for null count aggregation only or not. */
+  bool null_count_aggregate_only(const std::string& name) const {
+    if (!aggregate_only(name)) {
+      return false;
+    }
+
+    auto& aggregates = aggregates_.at(name);
+    for (auto& aggregate : aggregates) {
+      if (!aggregate->aggregation_validity_only()) {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
   /**
    * Returns if we need to process partial timestamp condition for this
    * fragment.
@@ -301,9 +384,9 @@ class ReaderBase : public StrategyBase {
   bool process_partial_timestamps(FragmentMetadata& frag_meta) const;
 
   /**
-   * Deletes the tiles on the input attribute/dimension from the result tiles.
+   * Deletes the tiles on the input field from the result tiles.
    *
-   * @param name The attribute/dimension name.
+   * @param name The field name.
    * @param result_tiles The result tiles to delete from.
    * @param min_result_tile The minimum index to start clearing tiles at.
    * @return void
@@ -388,14 +471,14 @@ class ReaderBase : public StrategyBase {
   }
 
   /**
-   * Loads tile offsets for each attribute/dimension name into
+   * Loads tile offsets for each field name into
    * their associated element in `fragment_metadata_`.
    *
    * @param relevant_fragments List of relevant fragments.
-   * @param names The attribute/dimension names.
+   * @param names The field names.
    * @return Status
    */
-  Status load_tile_offsets(
+  void load_tile_offsets(
       const RelevantFragments& relevant_fragments,
       const std::vector<std::string>& names);
 
@@ -421,14 +504,25 @@ class ReaderBase : public StrategyBase {
   Status add_delete_timestamps_condition();
 
   /**
-   * Loads tile var sizes for each attribute/dimension name into
+   * Loads tile var sizes for each field name into
    * their associated element in `fragment_metadata_`.
    *
    * @param relevant_fragments List of relevant fragments.
-   * @param names The attribute/dimension names.
-   * @return Status
+   * @param names The field names.
    */
-  Status load_tile_var_sizes(
+  void load_tile_var_sizes(
+      const RelevantFragments& relevant_fragments,
+      const std::vector<std::string>& names);
+
+  /*
+   * Loads tile metadata for each field name into
+   * their associated element in `fragment_metadata_`. This is done for
+   * attributes with aggregates.
+   *
+   * @param relevant_fragments List of relevant fragments.
+   * @param names The field names.
+   */
+  void load_tile_metadata(
       const RelevantFragments& relevant_fragments,
       const std::vector<std::string>& names);
 
@@ -438,7 +532,7 @@ class ReaderBase : public StrategyBase {
    * @param subarray The subarray to load processed conditions for.
    * @return Status
    */
-  Status load_processed_conditions();
+  void load_processed_conditions();
 
   /**
    * Read and unfilter attribute tiles.
@@ -449,7 +543,7 @@ class ReaderBase : public StrategyBase {
    * @return Status.
    */
   Status read_and_unfilter_attribute_tiles(
-      const std::vector<std::string>& names,
+      const std::vector<NameToLoad>& names,
       const std::vector<ResultTile*>& result_tiles) const;
 
   /**
@@ -477,7 +571,7 @@ class ReaderBase : public StrategyBase {
    * @return Filtered data blocks.
    */
   std::vector<FilteredData> read_attribute_tiles(
-      const std::vector<std::string>& names,
+      const std::vector<NameToLoad>& names,
       const std::vector<ResultTile*>& result_tiles) const;
 
   /**
@@ -503,35 +597,39 @@ class ReaderBase : public StrategyBase {
    * Concurrently executes across each name in `names` and each result tile
    * in 'result_tiles'.
    *
-   * @param names The attribute/dimension names.
+   * @param names The field names.
    * @param result_tiles The retrieved tiles will be stored inside the
    *     `ResultTile` instances in this vector.
+   * @param validity_only Is the field read for validity only.
    * @return Filtered data blocks.
    */
   std::vector<FilteredData> read_tiles(
-      const std::vector<std::string>& names,
+      const std::vector<NameToLoad>& names,
       const std::vector<ResultTile*>& result_tiles) const;
 
   /**
-   * Filters the tiles on a particular attribute/dimension from all input
+   * Filters the tiles on a particular field from all input
    * fragments based on the tile info in `result_tiles`.
    *
-   * @param name Attribute/dimension whose tiles will be unfiltered.
+   * @param name Field whose tiles will be unfiltered.
+   * @param validity_only Unfilter for the validity tile only?
    * @param result_tiles Vector containing the tiles to be unfiltered.
    * @return Status
    */
   Status unfilter_tiles(
       const std::string& name,
+      const bool validity_only,
       const std::vector<ResultTile*>& result_tiles) const;
 
   /**
    * Unfilter a specific range of chunks in tile
    *
-   * @param name Attribute/dimension the tile belong to.
+   * @param name field the tile belong to.
+   * @param validity_only Unfilter the validity tile only.
    * @param tile Offsets tile to be unfiltered.
-   * @param var_size True if the attribute/dimension is var-sized, false
+   * @param var_size True if the field is var-sized, false
    * otherwise
-   * @param nullable True if the attribute/dimension is nullable, false
+   * @param nullable True if the field is nullable, false
    * otherwise
    * @param range_thread_idx Current range thread index.
    * @param num_range_threads Total number of range threads.
@@ -543,6 +641,7 @@ class ReaderBase : public StrategyBase {
    */
   Status unfilter_tile(
       const std::string& name,
+      const bool validity_only,
       ResultTile* const tile,
       const bool var_size,
       const bool nullable,
@@ -591,7 +690,7 @@ class ReaderBase : public StrategyBase {
    * https://github.com/TileDB-Inc/TileDB/issues/1053. For format version > 4,
    * a tile never stores coordinates
    *
-   * @param name Attribute/dimension the tile belongs to.
+   * @param name field the tile belongs to.
    * @param tile Tile to zip the coordinates if needed.
    * @return Status
    */
@@ -601,11 +700,12 @@ class ReaderBase : public StrategyBase {
    * Reads the chunk data of all tile buffers and stores them in a data
    * structure together with the offsets between them
    *
-   * @param name Attribute/dimension the tile belong to.
+   * @param name field the tile belong to.
+   * @param validity_only Is the field read for validity only.
    * @param tile Offsets tile to be unfiltered.
-   * @param var_size True if the attribute/dimension is var-sized, false
+   * @param var_size True if the field is var-sized, false
    * otherwise
-   * @param nullable True if the attribute/dimension is nullable, false
+   * @param nullable True if the field is nullable, false
    * otherwise
    * @param tile_chunk_data Tile/offsets tile chunk info, buffers and
    * offsets
@@ -618,6 +718,7 @@ class ReaderBase : public StrategyBase {
   tuple<Status, optional<uint64_t>, optional<uint64_t>, optional<uint64_t>>
   load_tile_chunk_data(
       const std::string& name,
+      const bool validity_only,
       ResultTile* const tile,
       const bool var_size,
       const bool nullable,
@@ -628,11 +729,12 @@ class ReaderBase : public StrategyBase {
   /**
    * Perform some necessary post-processing on a tile that was just unfiiltered
    *
-   * @param name Attribute/dimension the tile belong to.
+   * @param name field the tile belong to.
+   * @param validity_only Is the field read for validity only.
    * @param tile Offsets tile that was just unfiltered.
-   * @param var_size True if the attribute/dimension is var-sized, false
+   * @param var_size True if the field is var-sized, false
    * otherwise
-   * @param nullable True if the attribute/dimension is nullable, false
+   * @param nullable True if the field is nullable, false
    * otherwise
    * @param unfiltered_tile_size Size of the unfiltered tile buffer
    * @param unfiltered_tile_var_size Size of the unfiltered tile_var buffer
@@ -642,6 +744,7 @@ class ReaderBase : public StrategyBase {
    */
   Status post_process_unfiltered_tile(
       const std::string& name,
+      const bool validity_only,
       ResultTile* const tile,
       const bool var_size,
       const bool nullable) const;
