@@ -266,6 +266,17 @@ void ordered_dim_label_reader_from_capnp(
     OrderedDimLabelReader* reader,
     ThreadPool* compute_tp);
 
+Status query_to_capnp(
+    Query& query, capnp::Query::Builder* query_builder, const bool client_side);
+
+Status query_from_capnp(
+    const capnp::Query::Reader& query_reader,
+    const SerializationContext context,
+    void* buffer_start,
+    CopyState* const copy_state,
+    Query* const query,
+    ThreadPool* compute_tp);
+
 #endif
 
 }  // namespace serialization
