@@ -63,7 +63,6 @@
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_time.h"
 #include "tiledb/sm/misc/utils.h"
-#include "tiledb/sm/misc/uuid.h"
 #include "tiledb/sm/query/deletes_and_updates/serialization.h"
 #include "tiledb/sm/query/query.h"
 #include "tiledb/sm/query/update_value.h"
@@ -72,6 +71,7 @@
 #include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/sm/tile/tile.h"
+#include "tiledb/storage_format/uri/generate_uri.h"
 #include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <algorithm>
@@ -80,8 +80,7 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 /* ****************************** */
 /*   CONSTRUCTORS & DESTRUCTORS   */
@@ -151,13 +150,12 @@ Status StorageManagerCanonical::group_close_for_writes(Group* group) {
       group->group_uri(), *group->encryption_key(), group->unsafe_metadata()));
 
   // Store any changes required
-  if (group->changes_applied()) {
+  if (group->group_details()->is_modified()) {
     const URI& group_detail_folder_uri = group->group_detail_uri();
-    auto&& [st, group_detail_uri] = group->generate_detail_uri();
-    RETURN_NOT_OK(st);
+    auto group_detail_uri = group->generate_detail_uri();
     RETURN_NOT_OK(store_group_detail(
         group_detail_folder_uri,
-        group_detail_uri.value(),
+        group_detail_uri,
         group->group_details(),
         *group->encryption_key()));
   }
@@ -169,87 +167,6 @@ Status StorageManagerCanonical::group_close_for_writes(Group* group) {
   return Status::Ok();
 }
 
-std::tuple<
-    Status,
-    optional<shared_ptr<ArraySchema>>,
-    optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>,
-    optional<std::vector<shared_ptr<FragmentMetadata>>>>
-StorageManagerCanonical::load_array_schemas_and_fragment_metadata(
-    const ArrayDirectory& array_dir,
-    MemoryTracker* memory_tracker,
-    const EncryptionKey& enc_key) {
-  auto timer_se =
-      stats()->start_timer("sm_load_array_schemas_and_fragment_metadata");
-
-  // Load array schemas
-  auto&& [array_schema_latest, array_schemas_all] =
-      array_dir.load_array_schemas(enc_key);
-
-  const auto filtered_fragment_uris =
-      load_filtered_fragment_uris(array_schema_latest->dense(), array_dir);
-  const auto& meta_uris = array_dir.fragment_meta_uris();
-  const auto& fragments_to_load = filtered_fragment_uris.fragment_uris();
-
-  // Get the consolidated fragment metadatas
-  std::vector<shared_ptr<Tile>> fragment_metadata_tiles(meta_uris.size());
-  std::vector<std::vector<std::pair<std::string, uint64_t>>> offsets_vectors(
-      meta_uris.size());
-  auto status = parallel_for(compute_tp(), 0, meta_uris.size(), [&](size_t i) {
-    auto&& [st, tile_opt, offsets] =
-        load_consolidated_fragment_meta(meta_uris[i], enc_key);
-    RETURN_NOT_OK(st);
-    fragment_metadata_tiles[i] =
-        make_shared<Tile>(HERE(), std::move(*tile_opt));
-    offsets_vectors[i] = std::move(offsets.value());
-    return st;
-  });
-  RETURN_NOT_OK_TUPLE(status, nullopt, nullopt, nullopt);
-
-  // Get the unique fragment metadatas into a map.
-  std::unordered_map<std::string, std::pair<Tile*, uint64_t>> offsets;
-  for (uint64_t i = 0; i < offsets_vectors.size(); i++) {
-    for (auto& offset : offsets_vectors[i]) {
-      if (offsets.count(offset.first) == 0) {
-        offsets.emplace(
-            offset.first,
-            std::make_pair(fragment_metadata_tiles[i].get(), offset.second));
-      }
-    }
-  }
-
-  // Load the fragment metadata
-  auto&& [st_fragment_meta, fragment_metadata] = load_fragment_metadata(
-      memory_tracker,
-      array_schema_latest,
-      array_schemas_all,
-      enc_key,
-      fragments_to_load,
-      offsets);
-  RETURN_NOT_OK_TUPLE(st_fragment_meta, nullopt, nullopt, nullopt);
-
-  return {
-      Status::Ok(), array_schema_latest, array_schemas_all, fragment_metadata};
-}
-
-tuple<Status, optional<std::vector<shared_ptr<FragmentMetadata>>>>
-StorageManagerCanonical::array_load_fragments(
-    Array* array, const std::vector<TimestampedURI>& fragments_to_load) {
-  auto timer_se = stats()->start_timer("sm_array_load_fragments");
-
-  // Load the fragment metadata
-  std::unordered_map<std::string, std::pair<Tile*, uint64_t>> offsets;
-  auto&& [st_fragment_meta, fragment_metadata] = load_fragment_metadata(
-      array->memory_tracker(),
-      array->array_schema_latest_ptr(),
-      array->array_schemas_all(),
-      *array->encryption_key(),
-      fragments_to_load,
-      offsets);
-  RETURN_NOT_OK_TUPLE(st_fragment_meta, nullopt);
-
-  return {Status::Ok(), fragment_metadata};
-}
-
 Status StorageManagerCanonical::array_consolidate(
     const char* array_name,
     EncryptionType encryption_type,
@@ -286,6 +203,7 @@ Status StorageManagerCanonical::array_consolidate(
 
   if (!encryption_key_from_cfg.empty()) {
     encryption_key = encryption_key_from_cfg.c_str();
+    key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
     std::string encryption_type_from_cfg;
     bool found = false;
     encryption_type_from_cfg = config.get("sm.encryption_type", &found);
@@ -294,16 +212,9 @@ Status StorageManagerCanonical::array_consolidate(
     RETURN_NOT_OK(st);
     encryption_type = et.value();
 
-    if (EncryptionKey::is_valid_key_length(
+    if (!EncryptionKey::is_valid_key_length(
             encryption_type,
             static_cast<uint32_t>(encryption_key_from_cfg.size()))) {
-      const UnitTestConfig& unit_test_cfg = UnitTestConfig::instance();
-      if (unit_test_cfg.array_encryption_key_length.is_set()) {
-        key_length = unit_test_cfg.array_encryption_key_length.get();
-      } else {
-        key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
-      }
-    } else {
       encryption_key = nullptr;
       key_length = 0;
     }
@@ -349,6 +260,7 @@ Status StorageManagerCanonical::fragments_consolidate(
 
   if (!encryption_key_from_cfg.empty()) {
     encryption_key = encryption_key_from_cfg.c_str();
+    key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
     std::string encryption_type_from_cfg;
     bool found = false;
     encryption_type_from_cfg = config.get("sm.encryption_type", &found);
@@ -357,16 +269,9 @@ Status StorageManagerCanonical::fragments_consolidate(
     RETURN_NOT_OK(st);
     encryption_type = et.value();
 
-    if (EncryptionKey::is_valid_key_length(
+    if (!EncryptionKey::is_valid_key_length(
             encryption_type,
             static_cast<uint32_t>(encryption_key_from_cfg.size()))) {
-      const UnitTestConfig& unit_test_cfg = UnitTestConfig::instance();
-      if (unit_test_cfg.array_encryption_key_length.is_set()) {
-        key_length = unit_test_cfg.array_encryption_key_length.get();
-      } else {
-        key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
-      }
-    } else {
       encryption_key = nullptr;
       key_length = 0;
     }
@@ -386,7 +291,7 @@ void StorageManagerCanonical::write_consolidated_commits_file(
     ArrayDirectory array_dir,
     const std::vector<URI>& commit_uris) {
   // Compute the file name.
-  auto name = array_dir.compute_new_fragment_name(
+  auto name = storage_format::generate_consolidated_fragment_name(
       commit_uris.front(), commit_uris.back(), write_version);
 
   // Compute size of consolidated file. Save the sizes of the files to re-use
@@ -585,6 +490,7 @@ Status StorageManagerCanonical::array_metadata_consolidate(
 
   if (!encryption_key_from_cfg.empty()) {
     encryption_key = encryption_key_from_cfg.c_str();
+    key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
     std::string encryption_type_from_cfg;
     bool found = false;
     encryption_type_from_cfg = config.get("sm.encryption_type", &found);
@@ -593,16 +499,9 @@ Status StorageManagerCanonical::array_metadata_consolidate(
     RETURN_NOT_OK(st);
     encryption_type = et.value();
 
-    if (EncryptionKey::is_valid_key_length(
+    if (!EncryptionKey::is_valid_key_length(
             encryption_type,
             static_cast<uint32_t>(encryption_key_from_cfg.size()))) {
-      const UnitTestConfig& unit_test_cfg = UnitTestConfig::instance();
-      if (unit_test_cfg.array_encryption_key_length.is_set()) {
-        key_length = unit_test_cfg.array_encryption_key_length.get();
-      } else {
-        key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
-      }
-    } else {
       encryption_key = nullptr;
       key_length = 0;
     }
@@ -634,7 +533,7 @@ Status StorageManagerCanonical::array_create(
 
   std::lock_guard<std::mutex> lock{object_create_mtx_};
   array_schema->set_array_uri(array_uri);
-  RETURN_NOT_OK(array_schema->generate_uri());
+  array_schema->generate_uri();
   array_schema->check(config_);
 
   // Create array directory
@@ -693,21 +592,10 @@ Status StorageManagerCanonical::array_create(
       RETURN_NOT_OK(
           encryption_key_cfg.set_key(encryption_type_cfg, nullptr, 0));
     } else {
-      uint32_t key_length = 0;
-      if (EncryptionKey::is_valid_key_length(
-              encryption_type_cfg,
-              static_cast<uint32_t>(encryption_key_from_cfg.size()))) {
-        const UnitTestConfig& unit_test_cfg = UnitTestConfig::instance();
-        if (unit_test_cfg.array_encryption_key_length.is_set()) {
-          key_length = unit_test_cfg.array_encryption_key_length.get();
-        } else {
-          key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
-        }
-      }
       RETURN_NOT_OK(encryption_key_cfg.set_key(
           encryption_type_cfg,
           (const void*)encryption_key_from_cfg.c_str(),
-          key_length));
+          static_cast<uint32_t>(encryption_key_from_cfg.size())));
     }
     st = store_array_schema(array_schema, encryption_key_cfg);
   } else {
@@ -821,34 +709,22 @@ Status StorageManagerCanonical::array_upgrade_version(
   if (encryption_key_from_cfg.empty()) {
     RETURN_NOT_OK(encryption_key_cfg.set_key(encryption_type_cfg, nullptr, 0));
   } else {
-    uint32_t key_length = 0;
-    if (EncryptionKey::is_valid_key_length(
-            encryption_type_cfg,
-            static_cast<uint32_t>(encryption_key_from_cfg.size()))) {
-      const UnitTestConfig& unit_test_cfg = UnitTestConfig::instance();
-      if (unit_test_cfg.array_encryption_key_length.is_set()) {
-        key_length = unit_test_cfg.array_encryption_key_length.get();
-      } else {
-        key_length = static_cast<uint32_t>(encryption_key_from_cfg.size());
-      }
-    }
     RETURN_NOT_OK(encryption_key_cfg.set_key(
         encryption_type_cfg,
         (const void*)encryption_key_from_cfg.c_str(),
-        key_length));
+        static_cast<uint32_t>(encryption_key_from_cfg.size())));
   }
 
   auto&& array_schema = array_dir.load_array_schema_latest(encryption_key_cfg);
 
   if (array_schema->version() < constants::format_version) {
-    auto st = array_schema->generate_uri();
-    RETURN_NOT_OK_ELSE(st, logger_->status_no_return_value(st));
+    array_schema->generate_uri();
     array_schema->set_version(constants::format_version);
 
     // Create array schema directory if necessary
     URI array_schema_dir_uri =
         array_uri.join_path(constants::array_schema_dir_name);
-    st = vfs()->create_dir(array_schema_dir_uri);
+    auto st = vfs()->create_dir(array_schema_dir_uri);
     RETURN_NOT_OK_ELSE(st, logger_->status_no_return_value(st));
 
     // Store array schema
@@ -1642,13 +1518,14 @@ Status StorageManagerCanonical::store_group_detail(
     tdb_shared_ptr<GroupDetails> group,
     const EncryptionKey& encryption_key) {
   // Serialize
+  auto members = group->members_to_serialize();
   SizeComputationSerializer size_computation_serializer;
-  group->serialize(size_computation_serializer);
+  group->serialize(members, size_computation_serializer);
 
   WriterTile tile{WriterTile::from_generic(size_computation_serializer.size())};
 
   Serializer serializer(tile.data(), tile.size());
-  group->serialize(serializer);
+  group->serialize(members, serializer);
 
   stats()->add_counter("write_group_size", tile.size());
 
@@ -1760,8 +1637,7 @@ Status StorageManagerCanonical::store_metadata(
   stats()->add_counter("write_meta_size", serializer.size());
 
   // Create a metadata file name
-  URI metadata_uri;
-  RETURN_NOT_OK(metadata->get_uri(uri, &metadata_uri));
+  URI metadata_uri = metadata->get_uri(uri);
 
   RETURN_NOT_OK(store_data_to_generic_tile(tile, metadata_uri, encryption_key));
 
@@ -1772,13 +1648,8 @@ Status StorageManagerCanonical::store_data_to_generic_tile(
     WriterTile& tile, const URI& uri, const EncryptionKey& encryption_key) {
   GenericTileIO tile_io(resources_, uri);
   uint64_t nbytes = 0;
-  Status st = tile_io.write_generic(&tile, encryption_key, &nbytes);
-
-  if (st.ok()) {
-    st = vfs()->close_file(uri);
-  }
-
-  return st;
+  tile_io.write_generic(&tile, encryption_key, &nbytes);
+  return vfs()->close_file(uri);
 }
 
 void StorageManagerCanonical::wait_for_zero_in_progress() {
@@ -1943,128 +1814,6 @@ void StorageManagerCanonical::load_group_metadata(
 /*         PRIVATE METHODS        */
 /* ****************************** */
 
-tuple<Status, optional<std::vector<shared_ptr<FragmentMetadata>>>>
-StorageManagerCanonical::load_fragment_metadata(
-    MemoryTracker* memory_tracker,
-    const shared_ptr<const ArraySchema>& array_schema_latest,
-    const std::unordered_map<std::string, shared_ptr<ArraySchema>>&
-        array_schemas_all,
-    const EncryptionKey& encryption_key,
-    const std::vector<TimestampedURI>& fragments_to_load,
-    const std::unordered_map<std::string, std::pair<Tile*, uint64_t>>&
-        offsets) {
-  auto timer_se = stats()->start_timer("sm_load_fragment_metadata");
-
-  // Load the metadata for each fragment
-  auto fragment_num = fragments_to_load.size();
-  std::vector<shared_ptr<FragmentMetadata>> fragment_metadata;
-  fragment_metadata.resize(fragment_num);
-  auto status = parallel_for(compute_tp(), 0, fragment_num, [&](size_t f) {
-    const auto& sf = fragments_to_load[f];
-
-    URI coords_uri =
-        sf.uri_.join_path(constants::coords + constants::file_suffix);
-
-    auto name = sf.uri_.remove_trailing_slash().last_path_part();
-    auto format_version = utils::parse::get_fragment_version(name);
-
-    // Note that the fragment metadata version is >= the array schema
-    // version. Therefore, the check below is defensive and will always
-    // ensure backwards compatibility.
-    shared_ptr<FragmentMetadata> metadata;
-    if (format_version <= 2) {
-      bool sparse;
-      RETURN_NOT_OK(vfs()->is_file(coords_uri, &sparse));
-      metadata = make_shared<FragmentMetadata>(
-          HERE(),
-          this,
-          memory_tracker,
-          array_schema_latest,
-          sf.uri_,
-          sf.timestamp_range_,
-          !sparse);
-    } else {
-      // Fragment format version > 2
-      metadata = make_shared<FragmentMetadata>(
-          HERE(),
-          this,
-          memory_tracker,
-          array_schema_latest,
-          sf.uri_,
-          sf.timestamp_range_);
-    }
-
-    // Potentially find the basic fragment metadata in the consolidated
-    // metadata buffer
-    Tile* fragment_metadata_tile = nullptr;
-    uint64_t offset = 0;
-
-    auto it = offsets.end();
-    if (metadata->format_version() >= 9) {
-      it = offsets.find(name);
-    } else {
-      it = offsets.find(sf.uri_.to_string());
-    }
-    if (it != offsets.end()) {
-      fragment_metadata_tile = it->second.first;
-      offset = it->second.second;
-    }
-
-    // Load fragment metadata
-    RETURN_NOT_OK(metadata->load(
-        encryption_key, fragment_metadata_tile, offset, array_schemas_all));
-
-    fragment_metadata[f] = metadata;
-    return Status::Ok();
-  });
-  RETURN_NOT_OK_TUPLE(status, nullopt);
-
-  return {Status::Ok(), fragment_metadata};
-}
-
-tuple<
-    Status,
-    optional<Tile>,
-    optional<std::vector<std::pair<std::string, uint64_t>>>>
-StorageManagerCanonical::load_consolidated_fragment_meta(
-    const URI& uri, const EncryptionKey& enc_key) {
-  auto timer_se = stats()->start_timer("sm_read_load_consolidated_frag_meta");
-
-  // No consolidated fragment metadata file
-  if (uri.to_string().empty())
-    return {Status::Ok(), nullopt, nullopt};
-
-  auto&& tile = GenericTileIO::load(resources_, uri, 0, enc_key);
-
-  stats()->add_counter("consolidated_frag_meta_size", tile.size());
-
-  uint32_t fragment_num;
-  Deserializer deserializer(tile.data(), tile.size());
-  fragment_num = deserializer.read<uint32_t>();
-
-  uint64_t name_size, offset;
-  std::string name;
-  std::vector<std::pair<std::string, uint64_t>> ret;
-  ret.reserve(fragment_num);
-  for (uint32_t f = 0; f < fragment_num; ++f) {
-    name_size = deserializer.read<uint64_t>();
-    name.resize(name_size);
-    deserializer.read(&name[0], name_size);
-    offset = deserializer.read<uint64_t>();
-    ret.emplace_back(name, offset);
-  }
-
-  return {Status::Ok(), std::move(tile), ret};
-}
-
-const ArrayDirectory::FilteredFragmentUris
-StorageManagerCanonical::load_filtered_fragment_uris(
-    const bool dense, const ArrayDirectory& array_dir) {
-  auto timer_se = stats()->start_timer("sm_load_filtered_fragment_uris");
-
-  return array_dir.filtered_fragment_uris(dense);
-}
-
 Status StorageManagerCanonical::set_default_tags() {
   const auto version = std::to_string(constants::library_version[0]) + "." +
                        std::to_string(constants::library_version[1]) + "." +
@@ -2124,5 +1873,4 @@ void StorageManagerCanonical::group_metadata_vacuum(
   consolidator->vacuum(group_name);
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
